# main.py
import machine
import neopixel
import time

# Pins
PIN_NP = 5
PIN_MODE= 0
PIN_COLOR = 4
# NeoPixel
NUM_LED = 16
# Mode
MODE_MAX = 5
MODES = [ "flow", "color_chase", "strobe", "on", "rainbow" ]
MODE_STEP = 1
# Color
COLOR_MAX = 255
COLOR_STEP = 3
# Brightness
BRIGHTNESS = 0.1
# Delay
DELAY = 0.1
DELAY_CHASE = 0.05
DELAY_STROBE = 0.4
DELAY_FLOW = 0.2
DELAY_MODE = 0.5

light = None
ts_mode = 0

def handle_mode(pin):
    global light, ts_mode
    if time.time() > (ts_mode + DELAY_MODE):
        ts_mode = time.time()
        light.set_fn("mode")

def handle_color(pin):
    global light
    light.set_fn("color")

class Light:
    def __init__(self, brightness):
        self.mode = 0
        self.color = 0
        self.brightness = brightness
        self.np = neopixel.NeoPixel(machine.Pin(PIN_NP), NUM_LED)
        self.sw_mode = machine.Pin(PIN_MODE, machine.Pin.IN, machine.Pin.PULL_UP)
        self.sw_color = machine.Pin(PIN_COLOR, machine.Pin.IN, machine.Pin.PULL_UP)
        self.sw_mode.irq(trigger=machine.Pin.IRQ_FALLING, handler=handle_mode)
        self.sw_color.irq(trigger=machine.Pin.IRQ_FALLING, handler=handle_color)

    def set_fn(self, fn):
        if fn == "mode":
            self.mode = (self.mode + MODE_STEP) % MODE_MAX
            print("set fn {} to {}".format(fn, MODES[self.mode]))
        elif fn == "color":
            self.color = (self.color + COLOR_STEP) % COLOR_MAX
            print("set fn {} to {}".format(fn, self.color))
        else:
            print("invalid fn {}".format(fn))

    def set_color(self, color):
        for i in range(NUM_LED):
            self.set_color_at(color, i)
        self.np.write()

    def set_color_at(self, color, index):
        c_tmp = list(self.wheel(color))
        for i in range(len(c_tmp)):
            c_tmp[i] = round(self.brightness * c_tmp[i])
        c = tuple(c_tmp)
        self.np[index] = c

    def apply(self):
        self.np.write()

    def wheel(self, pos):
        if pos < 0 or pos > 255:
           return (0, 0, 0)
        if pos < 85:
            return (255 - pos * 3, pos * 3, 0)
        if pos < 170:
            pos -= 85
            return (0, 255 - pos * 3, pos * 3)
        pos -= 170
        return (pos * 3, 0, 255 - pos * 3)

    def dim(self, color, brightness):
        pass


    def start(self):
        global state
        old_mode = None
        old_color = self.color
        i = 0
        j = 0
        while True:
            if not self.mode == old_mode:
                print("mode: {} ({}), old_mode: {}".format(self.mode, MODES[self.mode], old_mode))
            # On
            if MODES[self.mode] == "on":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    self.set_color(self.color)
                if not self.color == old_color:
                    old_color = self.color
                    self.set_color(self.color)
                time.sleep(DELAY)
            # Color Chase
            elif MODES[self.mode] == "color_chase":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    self.set_color(-1)
                    i = 0
                    j = 0
                    k = 0
                if not self.color == old_color:
                    old_color = self.color
                self.set_color_at(self.color, i)
                self.set_color_at(self.color, (i+1) % NUM_LED)
                self.set_color_at(self.color, (i+2) % NUM_LED)
                self.apply()
                self.set_color_at(-1, i)
                i = (i + 1) % NUM_LED
                time.sleep(DELAY_CHASE)
            # Flow
            elif MODES[self.mode] == "flow":
                if not self.mode == old_mode:
                    old_mode = self.mode
                self.color = (self.color + 1 ) % 255
                self.set_color(self.color)
                self.apply()
                time.sleep(DELAY_FLOW)
            # Strobe
            elif MODES[self.mode] == "strobe":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    j = 0
                if j == 1:
                    self.set_color(self.color)
                else:
                    self.set_color(-1)
                j = (j + 1) % 2
                time.sleep(DELAY_STROBE)
            # Rainbow
            elif MODES[self.mode] == "rainbow":
                if not self.mode == old_mode:
                    old_mode = self.mode
                    i = 0
                    j = 0
                rc_index = (i * 256 // NUM_LED) + j
                self.set_color_at(rc_index & 255, i)
                i = i + 1
                if i == NUM_LED:
                    i = 0
                    self.apply()
                    j = (j + 1) % 255
            else:
                print("else")


def main():
    print("")
    global light
    light = Light(BRIGHTNESS)
    light.start()


if __name__ == "__main__":
    main()